@@ -73,6 +73,9 @@
 #include <cmd_log.h>
 #include <cmd_fdos.h>
 #include <cmd_bmp.h>
+#include <cmd_portio.h>
+#include <cmd_mmc.h>
+#include <cmd_fat.h>
 
 #ifdef CONFIG_AMIGAONEG3SE
 #include <cmd_menu.h>
@@ -130,13 +133,14 @@ do_echo (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 
 		if (i > 1)
 			putc(' ');
-		while ((c = *p++) != '\0')
+		while ((c = *p++) != '\0') {
 			if (c == '\\' && *p == 'c') {
 				putnl = 0;
 				p++;
-			}
-			else
+			} else {
 				putc(c);
+			}
+		}
 	}
 
 	if (putnl)
@@ -189,8 +193,7 @@ do_help (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 			if (cmdtp->usage)
 				puts (cmdtp->usage);
 #endif	/* CFG_LONGHELP */
-		}
-		else {
+		} else {
 			printf ("Unknown command '%s' - try 'help'"
 				" without arguments for list of all"
 				" known commands\n\n",
@@ -262,6 +265,7 @@ cmd_tbl_t cmd_tbl[] = {
 	CMD_TBL_DTT
 	CMD_TBL_ECHO
 	CMD_TBL_EEPROM
+	CMD_TBL_FAT
 	CMD_TBL_FCCINFO
 	CMD_TBL_FLERASE
 	CMD_TBL_FDC
@@ -284,6 +288,7 @@ cmd_tbl_t cmd_tbl[] = {
 	CMD_TBL_IMM
 	CMD_TBL_INM
 	CMD_TBL_IMW
+	CMD_TBL_PORTIO_IN
 	CMD_TBL_ICRC
 	CMD_TBL_IPROBE
 	CMD_TBL_ILOOP
@@ -300,6 +305,7 @@ cmd_tbl_t cmd_tbl[] = {
 	CMD_TBL_LOOP
 	CMD_TBL_JFFS2_LS
 	CMD_TBL_MCCINFO
+	CMD_TBL_MMC
 	CMD_TBL_MD
 	CMD_TBL_MEMCINFO
 #ifdef CONFIG_AMIGAONEG3SE
@@ -314,6 +320,8 @@ cmd_tbl_t cmd_tbl[] = {
 	CMD_TBL_NANDBOOT
 	CMD_TBL_NEXT
 	CMD_TBL_NM
+	CMD_TBL_PING
+	CMD_TBL_PORTIO_OUT
 	CMD_TBL_PCI
 	CMD_TBL_PRINTENV
 	CMD_TBL_PROTECT
