@@ -319,13 +319,7 @@ void main_loop (void)
 	debug ("### main_loop entered: bootdelay=%d\n\n", bootdelay);
 
 # ifdef CONFIG_BOOT_RETRY_TIME
-	s = getenv ("bootretry");
-	if (s != NULL)
-		retry_time = (int)simple_strtoul(s, NULL, 10);
-	else
-		retry_time =  CONFIG_BOOT_RETRY_TIME;
-	if (retry_time >= 0 && retry_time < CONFIG_BOOT_RETRY_MIN)
-		retry_time = CONFIG_BOOT_RETRY_MIN;
+	init_cmd_timeout ();
 # endif	/* CONFIG_BOOT_RETRY_TIME */
 
 	s = getenv ("bootcmd");
@@ -422,10 +416,26 @@ void main_loop (void)
 #endif /*CFG_HUSH_PARSER*/
 }
 
+#ifdef CONFIG_BOOT_RETRY_TIME
+/***************************************************************************
+ * initialise command line timeout
+ */
+void init_cmd_timeout(void)
+{
+	char *s = getenv ("bootretry");
+
+	if (s != NULL)
+		retry_time = (int)simple_strtoul(s, NULL, 10);
+	else
+		retry_time =  CONFIG_BOOT_RETRY_TIME;
+
+	if (retry_time >= 0 && retry_time < CONFIG_BOOT_RETRY_MIN)
+		retry_time = CONFIG_BOOT_RETRY_MIN;
+}
+
 /***************************************************************************
  * reset command line timeout to retry_time seconds
  */
-#ifdef CONFIG_BOOT_RETRY_TIME
 void reset_cmd_timeout(void)
 {
 	endtime = endtick(retry_time);
@@ -746,9 +756,9 @@ int run_command (const char *cmd, int flag)
 	char finaltoken[CFG_CBSIZE];
 	char *str = cmdbuf;
 	char *argv[CFG_MAXARGS + 1];	/* NULL terminated	*/
-	int argc;
+	int argc, inquotes;
 	int repeatable = 1;
-	int inquotes;
+	int rc = 0;
 
 #ifdef DEBUG_PARSER
 	printf ("[RUN_COMMAND] cmd[%p]=\"", cmd);
@@ -817,13 +827,15 @@ int run_command (const char *cmd, int flag)
 		/* Look up command in command table */
 		if ((cmdtp = find_cmd(argv[0])) == NULL) {
 			printf ("Unknown command '%s' - try 'help'\n", argv[0]);
-			return -1;	/* give up after bad command */
+			rc = -1;	/* give up after bad command */
+			continue;
 		}
 
 		/* found - check max args */
 		if (argc > cmdtp->maxargs) {
 			printf ("Usage:\n%s\n", cmdtp->usage);
-			return -1;
+			rc = -1;
+			continue;
 		}
 
 #if (CONFIG_COMMANDS & CFG_CMD_BOOTD)
@@ -834,7 +846,8 @@ int run_command (const char *cmd, int flag)
 #endif
 			if (flag & CMD_FLAG_BOOTD) {
 				printf ("'bootd' recursion detected\n");
-				return -1;
+				rc = -1;
+				continue;
 			}
 			else
 				flag |= CMD_FLAG_BOOTD;
@@ -843,7 +856,7 @@ int run_command (const char *cmd, int flag)
 
 		/* OK - call function to do the command */
 		if ((cmdtp->cmd) (cmdtp, flag, argc, argv) != 0) {
-			return (-1);
+			rc = -1;
 		}
 
 		repeatable &= cmdtp->repeatable;
@@ -853,7 +866,7 @@ int run_command (const char *cmd, int flag)
 			return 0;	/* if stopped then not repeatable */
 	}
 
-	return repeatable;
+	return rc ? rc : repeatable;
 }
 
 /****************************************************************************/
@@ -880,7 +893,7 @@ int do_run (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
 			return 1;
 #else
 		if (parse_string_outer(arg,
-		    FLAG_PARSE_SEMICOLON | FLAG_EXIT_FROM_LOOP) == 0)
+		    FLAG_PARSE_SEMICOLON | FLAG_EXIT_FROM_LOOP) != 0)
 			return 1;
 #endif
 	}
