@@ -69,7 +69,15 @@
 #define CONFIG_MII		1	/* MII PHY management		*/
 #define	CONFIG_PHY_ADDR		0	/* PHY address			*/
 
+#if 0 /* test-only */
+#define CONFIG_BOOTP_MASK	(CONFIG_BOOTP_DEFAULT |  \
+				 CONFIG_BOOTP_VENDOREX)
+#else
+#define CONFIG_BOOTP_MASK       (CONFIG_BOOTP_DEFAULT)
+#endif
+
 #define CONFIG_COMMANDS	      ( CONFIG_CMD_DFL	| \
+				CFG_CMD_DHCP	| \
 				CFG_CMD_PCI	| \
 				CFG_CMD_IRQ	| \
 				CFG_CMD_IDE	| \
