@@ -24,7 +24,6 @@
 #include <net.h>
 #include "bootp.h"
 #include "tftp.h"
-#include "arp.h"
 #ifdef CONFIG_STATUS_LED
 #include <status_led.h>
 #endif
@@ -55,7 +54,7 @@ ulong		seed1, seed2;
 
 #if (CONFIG_COMMANDS & CFG_CMD_DHCP)
 dhcp_state_t dhcp_state = INIT;
-unsigned int dhcp_leasetime = 0;
+unsigned long dhcp_leasetime = 0;
 IPaddr_t NetDHCPServerIP = 0;
 static void DhcpHandler(uchar * pkt, unsigned dest, unsigned src, unsigned len);
 
@@ -336,10 +335,7 @@ BootpHandler(uchar * pkt, unsigned dest, unsigned src, unsigned len)
 		return;
 	}
 
-	/* Send ARP request to get TFTP server ethernet address.
-	 * This automagically starts TFTP, too.
-	 */
-	ArpRequest();
+	TftpStart();
 }
 #endif	/* !CFG_CMD_DHCP */
 
@@ -712,7 +708,7 @@ static void DhcpOptionsProcess(uchar *popt)
 				NetOurRootPath[size] = 0 ;
 				break;
 			case 51:
-				dhcp_leasetime = *(unsigned int *)(popt + 2);
+				NetCopyLong (&dhcp_leasetime, (ulong *)(popt + 2));
 				break;
 			case 53:		/* Ignore Message Type Option */
 				break;
@@ -866,10 +862,7 @@ DhcpHandler(uchar * pkt, unsigned dest, unsigned src, unsigned len)
 				NetState = NETLOOP_SUCCESS;
 				return;
 			}
-			/* Send ARP request to get TFTP server ethernet address.
-			 * This automagically starts TFTP, too.
-			 */
-			ArpRequest();
+			TftpStart();
 			return;
 		}
 		break;
